/***********************************************************
 * $Id: fancyqmidevice.h 1601 2015-03-13 08:32:22Z fancylibs $
 *
 * FancyLibs qmi wrapper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYQMIDEVICE_H_
#define FANCYQMIDEVICE_H_

#include <fancyqmi.h>
#include <fancyobject.h>

extern "C" {
typedef struct _QmiDevice GLibQmiDevice;
typedef struct _QmiClient QmiClient;
}

namespace Fancy {

class Runnable;

namespace QMI {

class QmiAsyncHandler;

/**
 * A handler for receiving results of asynchronous results.
 */
class FANCYQMI_API QmiDevice : virtual public Fancy::Object
{
	GLibQmiDevice *device;

public:
	typedef Fancy::ObjPtr<QmiDevice> ptr_t;

	/**
	 * Instantiate a new instance from a native glib handle.
	 * The preferred way to instantiate a QmiDevice is the
	 * asynchronous newInstance method below.
	 *
	 * The instance is dereferenced in the destructor but not
	 * additionally referenced in the constructor.
	 */
	static ptr_t newInstance(GLibQmiDevice *_device);

	/**
	 * Asynchronously create a new device instance.
	 * Please assure, that an instance of the main loop runnable
	 * returned by @c newMainLoop() is already running or will
	 * be started right after the call to @c newInstance().
	 *
	 * If no main loop is running, the passed async handler will
	 * not be informed about events.
	 *
	 * The preferred way to open a client for a device should be:
	 *
	 * \verbatim
	 * Fancy::ObjPtr<QmiAsyncHandler> myDeviceHandler = MyDeviceHandler::newInstance();
	 *
	 * int timeoutSeconds = 15;
	 * Fancy::ObjPtr<QmiAsyncHandler> openHandler =
	 *     QmiDevice::newOpenAndAllocateClientHandler(timeoutSeconds,myDeviceHandler);
	 *
	 * Fancy::ObjPtr<Fancy::Runnable> mainLoop = QmiDevice::newMainLoop();
	 *
	 * QmiDevice::newInstance("/dev/cdc-wdm1",openHandler);
	 *
	 * mainLoop->run();
	 *
	 * \endverbatim
	 *
	 * @param path The path to open.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<QmiDevice> instance.
	 */
	static void newInstance(const std::string &path, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * A main loop, which will handle the events of the asynchronous
	 * glib callbacks used to drive the QMI functionality.
	 */
	static Fancy::ObjPtr<Fancy::Runnable> newMainLoop();

	/**
	 * Return an integrate async handler, which opens a device and
	 * calls the given saync handler with the finally ready device.
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       device to be opened.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<QmiDevice> instance.
	 */
	static Fancy::ObjPtr<QmiAsyncHandler> newOpenHandler(int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &deviceResult);

	/**
	 * Open the device, this operation must be ready before
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       device to be opened.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<QmiDevice> instance.
	 */
	void open(int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Allocate a CMS (Device Management Service) client on top
	 * of an opened device.
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *               client to become available.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<QmiDmsClient> instance.
	 */
	void allocateDmsClient(int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Allocate an NAS (Network Availability Service) client on top
	 * of an opened device.
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *               client to become available.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<QmiNasClient> instance.
	 */
	void allocateNasClient(int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Register a client for release and log the result.
	 * The stop of the main loop will be postponed until
	 * all release operations are finished.
	 *
	 * @param client the native glib pointer to the client to be freed.
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *               client to become released.
	 */
	void releaseClient(QmiClient *client, int timeoutSeconds);

	std::string toString() const;

protected:

	QmiDevice(GLibQmiDevice *device);
	virtual ~QmiDevice();

};

}} // end namespaces

#endif
