/***********************************************************
 * $Id: fancyqminasclient.h 1617 2015-03-17 13:09:03Z fancylibs $
 *
 * FancyLibs qmi wrapper
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYQMINASCLIENT_H_
#define FANCYQMINASCLIENT_H_

#include <fancyqmienums.h>
#include <fancyobject.h>

extern "C" {
typedef struct _QmiClientNas QmiClientNas;
}

namespace Fancy {

template<class T> class ArrayObject;
class Number;

namespace QMI {

class QmiDevice;
class QmiAsyncHandler;
class QmiNetworkStatusCallback;
class QmiNetworkTimeCallback;
class QmiSignalInfoCallback;
class QmiSystemInfoCallback;

/**
 * A client operating on the Network Availability Service of a device.
 *
 * A client instance may not be directly instantiated, it
 * is returned as the result of an @c QmiDevice::allocateNasClient()
 * asynchronous operation.
 */
class FANCYQMI_API QmiNasClient : virtual public Fancy::Object
{
	Fancy::ObjPtr<QmiDevice> device;
	QmiClientNas *clientNas;

public:
	typedef Fancy::ObjPtr<QmiNasClient> ptr_t;

	/**
	 * Instantiate from a native glib instance.
	 * The instance is dereferenced in the destructor but not
	 * additionally referenced in the constructor.
	 */
	static ptr_t newInstance(const Fancy::ObjPtr<QmiDevice> &device, QmiClientNas *clientNas);

	/**
	 * Set a network technology preference. This call is deprecated, use
	 * setRatNodePreference() instead.
	 *
	 * @param radioTechnologyPreference Preference for the radio technology.
	 * @param duration the duration for which the new preference will be in effect.
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       preference to be propagated.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<Fancy::Boolean> instance.
	 */
	void setNetworkPreference(
			RadioTechnologyPreference radioTechnologyPreference,
			PreferenceDuration duration,
			int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Set a a radio technology preference by combining the enum
	 * constants of the RatModePreference enum. The applied setting
	 * will be permanent.
	 *
	 * @param ratModePreferenceMask Preference for the radio technology, like
	 *             RatModePreference_GSM|RatModePreference_UMTS|RatModePreference_LTE
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       preference to be propagated.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<Fancy::Boolean> instance.
	 */
	void setRatModePreferenceMask(int ratModePreferenceMask, int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Query the signal info on our own initiative.
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       signal info to become available.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<Fancy::QMI:QmiSignalInfo> instance.
	 */
	void getSignalInfo(int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Poll the signal info in regular intervals and propagate the received status to
	 * the given callback.
	 *
	 * @param pollIntervalSeconds The number of seconds to wait after a response from the
	 *           modem to perform another turn.
	 * @param timeoutSeconds timeout for the individual callbacks issued.
	 */
	void pollSignalInfo(int pollIntervalSeconds, int timeoutSeconds, const Fancy::ObjPtr<QmiSignalInfoCallback> &cb);

	/**
	 * Query the system info on our own initiative.
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       signal info to become available.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<Fancy::QMI:QmiSystemInfo> instance.
	 */
	void getSystemInfo(int timeoutSeconds, const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Poll the system info in regular intervals and propagate the received status to
	 * the given callback.
	 *
	 * @param pollIntervalSeconds The number of seconds to wait after a response from the
	 *           modem to perform another turn.
	 * @param timeoutSeconds timeout for the individual callbacks issued.
	 */
	void pollSystemInfo(int pollIntervalSeconds, int timeoutSeconds, const Fancy::ObjPtr<QmiSystemInfoCallback> &cb);

	/**
	 * Initiate unsolicited network status callbacks and set the thresholds
	 * to use for signal strength reports.
	 *
	 * @param signalStrengthThresholds An array of thresholds, which trigger an actual
	 *               network status callback. If null, network status reporting is
	 *               turned off. Network status callbacks are deprecated, so
	 *               set this parameter to null for most situations.
	 * @param threshold2G A threshold for new-style signal info callbacks, in dB.
	 *                    Values around 3 or 5dB are recommended. This value
	 *                    affects rssi values for basic 2G reporting.
	 * @param threshold3G A threshold for new-style signal info callbacks, in dB.
	 *                    Values around 3 or 5dB are recommended. This value
	 *                    affects ecio and snr values for 3G reporting.
	 * @param thresholdLTE A threshold for new-style signal info callbacks, in dB.
	 *                     Values around 3 or 5dB are recommended. This value
	 *                     affects rsrp and rsrq values for 4G/LTE reporting.
	 *                     Setting this value to non-null might up in a messaging
	 *                     error, so set it only, if you really need LTE signal infos.
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *                       preferencees to be propagated.
	 * @param result An asynchronous result which receives a
	 *               Fancy::ObjPtr<Fancy::Boolean> instance.
	 */
	void setNetworkStatusPreferences(
			const Fancy::ObjPtr<Fancy::ArrayObject<int32_t> > &signalStrengthThresholds,
			const Fancy::ObjPtr<Fancy::Number> &threshold2G,
			const Fancy::ObjPtr<Fancy::Number> &threshold3G,
			const Fancy::ObjPtr<Fancy::Number> &thresholdLTE,
			int timeoutSeconds,
			const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Activate unsolicited callbacks.
	 *
	 * @param reportSystemInfo Whether to activate system info callbacks.
	 * @param reportSignalInfo Whether to activate signal info callbacks.
	 * @param reportNetworkTime Whether to activate network time callbacks.
	 */
	void setEventPreferences(bool reportSystemInfo, bool reportSignalInfo, bool reportNetworkTime,
			int timeoutSeconds,
			const Fancy::ObjPtr<QmiAsyncHandler> &result);

	/**
	 * Register a callback for network status events from the device.
	 * Network status events are deprecated, use the new-style signal info
	 * callback instead.
	 */
	void setNetworkStatusCallback(const Fancy::ObjPtr<QmiNetworkStatusCallback> &cb);

	/**
	 * Register a callback for network time events from the device.
	 */
	void setNetworkTimeCallback(const Fancy::ObjPtr<QmiNetworkTimeCallback> &cb);

	/**
	 * Register a callback for signal info events from the device.
	 */
	void setSignalInfoCallback(const Fancy::ObjPtr<QmiSignalInfoCallback> &cb);

	/**
	 * Register a callback for system info events from the device.
	 */
	void setSystemInfoCallback(const Fancy::ObjPtr<QmiSystemInfoCallback> &cb);

	/**
	 * Register this client for release at the underlying device.
	 * The stop of the main loop will be postponed until
	 * all release operations are finished.
	 *
	 * If not performed explicitly, this operation is performed with
	 * a timeout of 1 seconds by the destructor.
	 *
	 * This operation renders the client instance unusable and set the
	 * kept device pointer to null.
	 *
	 * @param timeoutSeconds maximum time, in seconds, to wait for the
	 *               client to become released.
	 */
	void release(int timeoutSeconds);

	std::string toString() const;

protected:

	QmiNasClient(const Fancy::ObjPtr<QmiDevice> &device, QmiClientNas *clientNas);
	virtual ~QmiNasClient();

};

}} // end namespaces

#endif
